@@ -14,13 +14,14 @@ DEFSECTION = "Poezio"
 
 import logging.config
 import os
+import stat
 import sys
 import pkg_resources
 
 from configparser import RawConfigParser, NoOptionError, NoSectionError
 from os import environ, makedirs, path, remove
 from shutil import copy2
-from args import parse_args
+from poezio.args import parse_args
 
 DEFAULT_CONFIG = {
     'Poezio': {
@@ -36,8 +37,8 @@ DEFAULT_CONFIG = {
         'certificate': '',
         'certfile': '',
         'ciphers': 'HIGH+kEDH:HIGH+kEECDH:HIGH:!PSK:!SRP:!3DES:!aNULL',
-        'connection_check_interval': 60,
-        'connection_timeout_delay': 10,
+        'connection_check_interval': 300,
+        'connection_timeout_delay': 30,
         'create_gaps': False,
         'custom_host': '',
         'custom_port': '',
@@ -49,7 +50,7 @@ DEFAULT_CONFIG = {
         'display_mood_notifications': False,
         'display_tune_notifications': False,
         'display_user_color_in_join_part': True,
-        'enable_carbons': False,
+        'enable_carbons': True,
         'enable_user_activity': True,
         'enable_user_gaming': True,
         'enable_user_mood': True,
@@ -57,6 +58,8 @@ DEFAULT_CONFIG = {
         'enable_user_tune': True,
         'enable_vertical_tab_list': False,
         'enable_xhtml_im': True,
+        'enable_smacks': False,
+        'eval_password': '',
         'exec_remote': False,
         'extract_inline_images': True,
         'filter_info_messages': '',
@@ -71,6 +74,7 @@ DEFAULT_CONFIG = {
         'ignore_certificate': False,
         'ignore_private': False,
         'information_buffer_popup_on': 'error roster warning help info',
+        'information_buffer_type_filter': '',
         'jid': '',
         'keyfile': '',
         'lang': 'en',
@@ -92,7 +96,6 @@ DEFAULT_CONFIG = {
         'private_auto_response': '',
         'remote_fifo_path': './',
         'request_message_receipts': True,
-        'resource': '',
         'rooms': '',
         'roster_group_sort': 'name',
         'roster_show_offline': False,
@@ -116,7 +119,7 @@ DEFAULT_CONFIG = {
         'show_tab_names': False,
         'show_tab_numbers': True,
         'show_timestamps': True,
-        'show_useless_separator': False,
+        'show_useless_separator': True,
         'status': '',
         'status_message': '',
         'theme': 'default',
@@ -156,10 +159,7 @@ class Config(RawConfigParser):
         self.default = default
 
     def read_file(self):
-        try:
-            RawConfigParser.read(self, self.file_name, encoding='utf-8')
-        except TypeError: # python < 3.2 sucks
-            RawConfigParser.read(self, self.file_name)
+        RawConfigParser.read(self, self.file_name, encoding='utf-8')
         # Check config integrity and fix it if it’s wrong
         # only when the object is the main config
         if self.__class__ is Config:
@@ -563,6 +563,14 @@ def run_cmdline_args(CONFIG_PATH):
             copy2(default, options.filename)
         elif path.isfile(other):
             copy2(other, options.filename)
+
+        # Inside the nixstore and possibly other distributions, the reference
+        # file is readonly, so is the copy.
+        # Make it writable by the user who just created it.
+        if os.path.exists(options.filename):
+            os.chmod(options.filename,
+                     os.stat(options.filename).st_mode | stat.S_IWUSR)
+
         global firstrun
         firstrun = True
 
@@ -609,6 +617,7 @@ def setup_logging():
                 'filename': path.join(LOG_DIR, 'errors.log'),
                 'formatter': 'simple',
             }
+        logging.disable(logging.WARNING)
 
     if options.debug:
         LOGGING_CONFIG['root']['handlers'].append('debug')
@@ -618,11 +627,13 @@ def setup_logging():
                 'filename': options.debug,
                 'formatter': 'simple',
             }
+        logging.disable(logging.NOTSET)
 
 
     if LOGGING_CONFIG['root']['handlers']:
         logging.config.dictConfig(LOGGING_CONFIG)
     else:
+        logging.disable(logging.ERROR)
         logging.basicConfig(level=logging.CRITICAL)
 
     global log
@@ -631,7 +642,7 @@ def setup_logging():
 def post_logging_setup():
     # common imports slixmpp, which creates then its loggers, so
     # it needs to be after logger configuration
-    from common import safeJID as JID
+    from poezio.common import safeJID as JID
     global safeJID
     safeJID = JID
 
