@@ -16,24 +16,22 @@ revolving around chats.
 import logging
 log = logging.getLogger(__name__)
 
-import singleton
 import string
 import time
-import weakref
-from datetime import datetime, timedelta
+from datetime import datetime
 from xml.etree import cElementTree as ET
 
-import core
-import timed_events
-import windows
-import xhtml
-from common import safeJID
-from config import config
-from decorators import refresh_wrapper
-from logger import logger
-from text_buffer import TextBuffer
-from theming import get_theme, dump_tuple
-from decorators import command_args_parser
+from poezio.core.structs import Command, Completion, Status
+from poezio import timed_events
+from poezio import windows
+from poezio import xhtml
+from poezio.common import safeJID
+from poezio.config import config
+from poezio.decorators import refresh_wrapper
+from poezio.logger import logger
+from poezio.text_buffer import TextBuffer
+from poezio.theming import get_theme, dump_tuple
+from poezio.decorators import command_args_parser
 
 # getters for tab colors (lambdas, so that they are dynamic)
 STATE_COLORS = {
@@ -72,24 +70,31 @@ STATE_PRIORITY = {
         'disconnected': 0,
         'nonempty': 0.1,
         'scrolled': 0.5,
+        'joined': 0.8,
         'composing': 0.9,
         'message': 1,
-        'joined': 1,
         'highlight': 2,
         'private': 2,
         'attention': 3
     }
 
-class Tab(object):
-    tab_core = None
-    size_manager = None
+SHOW_NAME = {
+        'dnd': 'busy',
+        'away': 'away',
+        'xa': 'not available',
+        'chat': 'chatty',
+        '': 'available'
+    }
 
+class Tab(object):
     plugin_commands = {}
     plugin_keys = {}
-    def __init__(self):
+    def __init__(self, core):
+        self.core = core
         if not hasattr(self, 'name'):
             self.name = self.__class__.__name__
         self.input = None
+        self.closed = False
         self._state = 'normal'
         self._prev_state = None
 
@@ -101,15 +106,7 @@ class Tab(object):
 
     @property
     def size(self):
-        if not Tab.size_manager:
-            Tab.size_manager = self.core.size
-        return Tab.size_manager
-
-    @property
-    def core(self):
-        if not Tab.tab_core:
-            Tab.tab_core = singleton.Singleton(core.Core)
-        return Tab.tab_core
+        return self.core.size
 
     @property
     def nb(self):
@@ -118,18 +115,6 @@ class Tab(object):
                 return index
         return len(self.core.tabs)
 
-    @property
-    def tab_win(self):
-        if not Tab.tab_core:
-            Tab.tab_core = singleton.Singleton(core.Core)
-        return Tab.tab_core.tab_win
-
-    @property
-    def left_tab_win(self):
-        if not Tab.tab_core:
-            Tab.tab_core = singleton.Singleton(core.Core)
-        return Tab.tab_core.left_tab_win
-
     @staticmethod
     def tab_win_height():
         """
@@ -188,7 +173,7 @@ class Tab(object):
     @staticmethod
     def resize(scr):
         Tab.height, Tab.width = scr.getmaxyx()
-        windows.Win._tab_win = scr
+        windows.base_wins.TAB_WIN = scr
 
     def missing_command_callback(self, command_name):
         """
@@ -206,7 +191,7 @@ class Tab(object):
             return
         if not desc and shortdesc:
             desc = shortdesc
-        self.commands[name] = core.Command(func, desc, completion, shortdesc, usage)
+        self.commands[name] = Command(func, desc, completion, shortdesc, usage)
 
     def complete_commands(self, the_input):
         """
@@ -251,10 +236,13 @@ class Tab(object):
                     command = self.core.commands[command_name]
                 else:           # Unknown command, cannot complete
                     return False
-                if command[2] is None:
+                if command.comp is None:
                     return False # There's no completion function
                 else:
-                    return command[2](the_input)
+                    comp = command.comp(the_input)
+                    if comp:
+                        return comp.run()
+                    return comp
         return False
 
     def execute_command(self, provided_text):
@@ -269,15 +257,15 @@ class Tab(object):
             arg = txt[2+len(command):] # jump the '/' and the ' '
             func = None
             if command in self.commands: # check tab-specific commands
-                func = self.commands[command][0]
+                func = self.commands[command].func
             elif command in self.core.commands: # check global commands
-                func = self.core.commands[command][0]
+                func = self.core.commands[command].func
             else:
                 low = command.lower()
                 if low in self.commands:
-                    func = self.commands[low][0]
+                    func = self.commands[low].func
                 elif low in self.core.commands:
-                    func = self.core.commands[low][0]
+                    func = self.core.commands[low].func
                 else:
                     if self.missing_command_callback is not None:
                         error_handled = self.missing_command_callback(low)
@@ -290,7 +278,8 @@ class Tab(object):
             else:
                 arg = xhtml.clean_text_simple(arg)
             if func:
-                self.input.reset_completion()
+                if hasattr(self.input, "reset_completion"):
+                    self.input.reset_completion()
                 func(arg)
             return True
         else:
@@ -298,10 +287,11 @@ class Tab(object):
 
     def refresh_tab_win(self):
         if config.get('enable_vertical_tab_list'):
-            if self.left_tab_win and not self.size.core_degrade_x:
-                self.left_tab_win.refresh()
+            left_tab_win = self.core.left_tab_win
+            if left_tab_win and not self.size.core_degrade_x:
+                left_tab_win.refresh()
         elif not self.size.core_degrade_y:
-            self.tab_win.refresh()
+            self.core.tab_win.refresh()
 
     def refresh(self):
         """
@@ -393,7 +383,7 @@ class Tab(object):
 
     def on_info_win_size_changed(self):
         """
-        Called when the window with the informations is resized
+        Called when the window with the information is resized
         """
         pass
 
@@ -403,6 +393,7 @@ class Tab(object):
         """
         if self.input:
             self.input.on_delete()
+        self.closed = True
 
     def matching_names(self):
         """
@@ -440,8 +431,8 @@ class ChatTab(Tab):
     """
     plugin_commands = {}
     plugin_keys = {}
-    def __init__(self, jid=''):
-        Tab.__init__(self)
+    def __init__(self, core, jid=''):
+        Tab.__init__(self, core)
         self.name = jid
         self.text_win = None
         self._text_buffer = TextBuffer()
@@ -645,16 +636,16 @@ class ChatTab(Tab):
         /correct <fixed message>
         """
         if not line:
-            self.core.command_help('correct')
+            self.core.command.help('correct')
             return
         if not self.last_sent_message:
-            self.core.information('There is no message to correct.')
+            self.core.information('There is no message to correct.', 'Error')
             return
         self.command_say(line, correct=True)
 
     def completion_correct(self, the_input):
         if self.last_sent_message and the_input.get_argument_position() == 1:
-            return the_input.auto_completion([self.last_sent_message['body']], '', quotify=False)
+            return Completion(the_input.auto_completion, [self.last_sent_message['body']], '', quotify=False)
 
     @property
     def inactive(self):
@@ -703,8 +694,11 @@ class ChatTab(Tab):
 
 class OneToOneTab(ChatTab):
 
-    def __init__(self, jid=''):
-        ChatTab.__init__(self, jid)
+    def __init__(self, core, jid=''):
+        ChatTab.__init__(self, core, jid)
+
+        self.__status = Status("", "")
+        self.last_remote_message = datetime.now()
 
         # Set to true once the first disco is done
         self.__initial_disco = False
@@ -716,6 +710,33 @@ class OneToOneTab(ChatTab):
         self.remote_supports_receipts = True
         self.check_features()
 
+    def remote_user_color(self):
+        return dump_tuple(get_theme().COLOR_REMOTE_USER)
+
+    def update_status(self, status):
+        old_status = self.__status
+        if not (old_status.show != status.show or
+                old_status.message != status.message):
+            return
+        self.__status = status
+        hide_status_change = config.get_by_tabname('hide_status_change',
+                                                   safeJID(self.name).bare)
+        now = datetime.now()
+        dff = now - self.last_remote_message
+        if hide_status_change > -1 and dff.total_seconds() > hide_status_change:
+            return
+
+        info_c = dump_tuple(get_theme().COLOR_INFORMATION_TEXT)
+        nick = self.get_nick()
+        remote = self.remote_user_color()
+        msg = '\x19%(color)s}%(nick)s\x19%(info)s} changed: '
+        msg %= {'color': remote, 'nick': nick, 'info': info_c}
+        if status.message != old_status.message and status.message:
+            msg += 'status: %s, ' % status.message
+        if status.show in SHOW_NAME:
+            msg += 'show: %s, ' % SHOW_NAME[status.show]
+        self.add_message(msg[:-2], typ=2)
+
     @property
     def remote_wants_chatstates(self):
         return self._remote_wants_chatstates
@@ -762,6 +783,7 @@ class OneToOneTab(ChatTab):
     def command_xhtml(self, xhtml_data):
         message = self.generate_xhtml_message(xhtml_data)
         if message:
+            message['type'] = 'chat'
             if self.remote_supports_receipts:
                 message._add_receipt = True
             if self.remote_wants_chatstates:
@@ -874,4 +896,3 @@ class OneToOneTab(ChatTab):
         self.add_message(msg, typ=0)
         self.core.refresh_window()
 
-
