@@ -16,18 +16,18 @@ log = logging.getLogger(__name__)
 
 import curses
 
-from . basetabs import OneToOneTab, Tab
+from poezio.tabs.basetabs import OneToOneTab, Tab
 
-import common
-import fixes
-import windows
-import xhtml
-from common import safeJID
-from config import config
-from decorators import refresh_wrapper
-from roster import roster
-from theming import get_theme, dump_tuple
-from decorators import command_args_parser
+from poezio import common
+from poezio import fixes
+from poezio import windows
+from poezio import xhtml
+from poezio.common import safeJID
+from poezio.config import config
+from poezio.decorators import refresh_wrapper
+from poezio.roster import roster
+from poezio.theming import get_theme, dump_tuple
+from poezio.decorators import command_args_parser
 
 class ConversationTab(OneToOneTab):
     """
@@ -36,10 +36,10 @@ class ConversationTab(OneToOneTab):
     """
     plugin_commands = {}
     plugin_keys = {}
-    additional_informations = {}
+    additional_information = {}
     message_type = 'chat'
-    def __init__(self, jid):
-        OneToOneTab.__init__(self, jid)
+    def __init__(self, core, jid):
+        OneToOneTab.__init__(self, core, jid)
         self.nick = None
         self.nick_sent = False
         self.state = 'normal'
@@ -64,7 +64,7 @@ class ConversationTab(OneToOneTab):
                 usage='[jid]',
                 desc='Get the last activity of the given or the current contact.',
                 shortdesc='Get the activity.',
-                completion=self.core.completion_last_activity)
+                completion=self.core.completion.last_activity)
         self.resize()
         self.update_commands()
         self.update_keys()
@@ -78,11 +78,11 @@ class ConversationTab(OneToOneTab):
         """
         Lets a plugin add its own information to the ConversationInfoWin
         """
-        ConversationTab.additional_informations[plugin_name] = callback
+        ConversationTab.additional_information[plugin_name] = callback
 
     @staticmethod
     def remove_information_element(plugin_name):
-        del ConversationTab.additional_informations[plugin_name]
+        del ConversationTab.additional_information[plugin_name]
 
     def completion(self):
         self.complete_commands(self.input)
@@ -155,7 +155,7 @@ class ConversationTab(OneToOneTab):
         /last_activity [jid]
         """
         if args and args[0]:
-            return self.core.command_last_activity(args[0])
+            return self.core.command.last_activity(args[0])
 
         def callback(iq):
             if iq['type'] != 'result':
@@ -207,7 +207,7 @@ class ConversationTab(OneToOneTab):
 
     @command_args_parser.ignored
     def command_unquery(self):
-        self.core.close_tab()
+        self.core.close_tab(self)
 
     @command_args_parser.quoted(0, 1)
     def command_version(self, args):
@@ -223,7 +223,7 @@ class ConversationTab(OneToOneTab):
                                                              res.get('os') or 'an unknown platform')
             self.core.information(version, 'Info')
         if args:
-            return self.core.command_version(args[0])
+            return self.core.command.version(args[0])
         jid = safeJID(self.name)
         if not jid.resource:
             if jid in roster:
@@ -267,7 +267,7 @@ class ConversationTab(OneToOneTab):
 
         if display_bar:
             self.upper_bar.refresh(self.get_dest_jid(), roster[self.get_dest_jid()])
-        self.info_header.refresh(self.get_dest_jid(), roster[self.get_dest_jid()], self.text_win, self.chatstate, ConversationTab.additional_informations)
+        self.info_header.refresh(self.get_dest_jid(), roster[self.get_dest_jid()], self.text_win, self.chatstate, ConversationTab.additional_information)
 
         if display_info_win:
             self.info_win.refresh()
@@ -276,7 +276,7 @@ class ConversationTab(OneToOneTab):
 
     def refresh_info_header(self):
         self.info_header.refresh(self.get_dest_jid(), roster[self.get_dest_jid()],
-                self.text_win, self.chatstate, ConversationTab.additional_informations)
+                self.text_win, self.chatstate, ConversationTab.additional_information)
         self.input.refresh()
 
     def get_nick(self):
@@ -370,13 +370,13 @@ class DynamicConversationTab(ConversationTab):
     a full jid, and unlocked, as described in the XEP-0296.
     Only one DynamicConversationTab can be opened for a given jid.
     """
-    def __init__(self, jid, resource=None):
+    def __init__(self, core, jid, resource=None):
         self.locked_resource = None
         self.name = safeJID(jid).bare
         if resource:
             self.lock(resource)
         self.info_header = windows.DynamicConversationInfoWin()
-        ConversationTab.__init__(self, jid)
+        ConversationTab.__init__(self, core, jid)
         self.register_command('unlock', self.unlock_command,
                 shortdesc='Unlock the conversation from a particular resource.')
 
@@ -452,7 +452,7 @@ class DynamicConversationTab(ConversationTab):
             displayed_jid = self.name
         self.info_header.refresh(displayed_jid, roster[self.name],
                                  self.text_win, self.chatstate,
-                                 ConversationTab.additional_informations)
+                                 ConversationTab.additional_information)
         if display_info_win:
             self.info_win.refresh()
 
@@ -468,7 +468,7 @@ class DynamicConversationTab(ConversationTab):
         else:
             displayed_jid = self.name
         self.info_header.refresh(displayed_jid, roster[self.name],
-                self.text_win, self.chatstate, ConversationTab.additional_informations)
+                self.text_win, self.chatstate, ConversationTab.additional_information)
         self.input.refresh()
 
 class StaticConversationTab(ConversationTab):
@@ -476,9 +476,8 @@ class StaticConversationTab(ConversationTab):
     A conversation tab associated with one Full JID. It cannot be locked to
     an different resource or unlocked.
     """
-    def __init__(self, jid):
+    def __init__(self, core, jid):
         assert(safeJID(jid).resource)
         self.info_header = windows.ConversationInfoWin()
-        ConversationTab.__init__(self, jid)
-
+        ConversationTab.__init__(self, core, jid)
 
