@@ -17,31 +17,24 @@ import random
 import re
 from datetime import datetime
 
-from . import ChatTab, Tab
+from poezio.tabs import ChatTab, Tab, SHOW_NAME
 
-import common
-import fixes
-import multiuserchat as muc
-import timed_events
-import windows
-import xhtml
-from common import safeJID
-from config import config
-from decorators import refresh_wrapper, command_args_parser
-from logger import logger
-from roster import roster
-from theming import get_theme, dump_tuple
-from user import User
+from poezio import common
+from poezio import fixes
+from poezio import multiuserchat as muc
+from poezio import timed_events
+from poezio import windows
+from poezio import xhtml
+from poezio.common import safeJID
+from poezio.config import config
+from poezio.decorators import refresh_wrapper, command_args_parser
+from poezio.logger import logger
+from poezio.roster import roster
+from poezio.theming import get_theme, dump_tuple
+from poezio.user import User
+from poezio.core.structs import Completion, Status
 
 
-SHOW_NAME = {
-    'dnd': 'busy',
-    'away': 'away',
-    'xa': 'not available',
-    'chat': 'chatty',
-    '': 'available'
-    }
-
 NS_MUC_USER = 'http://jabber.org/protocol/muc#user'
 
 
@@ -53,12 +46,13 @@ class MucTab(ChatTab):
     message_type = 'groupchat'
     plugin_commands = {}
     plugin_keys = {}
-    def __init__(self, jid, nick, password=None):
+    def __init__(self, core, jid, nick, password=None):
         self.joined = False
-        ChatTab.__init__(self, jid)
+        ChatTab.__init__(self, core, jid)
         if self.joined == False:
             self._state = 'disconnected'
         self.own_nick = nick
+        self.own_user = None
         self.name = jid
         self.password = password
         self.users = []
@@ -195,9 +189,6 @@ class MucTab(ChatTab):
                 shortdesc='Invite a contact to this room',
                 completion=self.completion_invite)
 
-        if self.core.xmpp.boundjid.server == "gmail.com": #gmail sucks
-            del self.commands["nick"]
-
         self.resize()
         self.update_commands()
         self.update_keys()
@@ -245,7 +236,7 @@ class MucTab(ChatTab):
         comp.sort()
         userlist.extend(comp)
 
-        return the_input.auto_completion(userlist, quotify=False)
+        return Completion(the_input.auto_completion, userlist, quotify=False)
 
     def completion_info(self, the_input):
         """Completion for /info"""
@@ -253,7 +244,7 @@ class MucTab(ChatTab):
         userlist = []
         for user in sorted(self.users, key=compare_users, reverse=True):
             userlist.append(user.nick)
-        return the_input.auto_completion(userlist, quotify=False)
+        return Completion(the_input.auto_completion, userlist, quotify=False)
 
     def completion_nick(self, the_input):
         """Completion for /nick"""
@@ -261,11 +252,11 @@ class MucTab(ChatTab):
                  config.get('default_nick'),
                  self.core.get_bookmark_nickname(self.name)]
         nicks = [i for i in nicks if i]
-        return the_input.auto_completion(nicks, '', quotify=False)
+        return Completion(the_input.auto_completion, nicks, '', quotify=False)
 
     def completion_recolor(self, the_input):
         if the_input.get_argument_position() == 1:
-            return the_input.new_completion(['random'], 1, '', quotify=False)
+            return Completion(the_input.new_completion, ['random'], 1, '', quotify=False)
         return True
 
     def completion_color(self, the_input):
@@ -275,13 +266,13 @@ class MucTab(ChatTab):
             userlist = [user.nick for user in self.users]
             if self.own_nick in userlist:
                 userlist.remove(self.own_nick)
-            return the_input.new_completion(userlist, 1, '', quotify=True)
+            return Completion(the_input.new_completion, userlist, 1, '', quotify=True)
         elif n == 2:
             colors = [i for i in xhtml.colors if i]
             colors.sort()
             colors.append('unset')
             colors.append('random')
-            return the_input.new_completion(colors, 2, '', quotify=False)
+            return Completion(the_input.new_completion, colors, 2, '', quotify=False)
 
     def completion_ignore(self, the_input):
         """Completion for /ignore"""
@@ -289,7 +280,7 @@ class MucTab(ChatTab):
         if self.own_nick in userlist:
             userlist.remove(self.own_nick)
         userlist.sort()
-        return the_input.auto_completion(userlist, quotify=False)
+        return Completion(the_input.auto_completion, userlist, quotify=False)
 
     def completion_role(self, the_input):
         """Completion for /role"""
@@ -298,10 +289,10 @@ class MucTab(ChatTab):
             userlist = [user.nick for user in self.users]
             if self.own_nick in userlist:
                 userlist.remove(self.own_nick)
-            return the_input.new_completion(userlist, 1, '', quotify=True)
+            return Completion(the_input.new_completion, userlist, 1, '', quotify=True)
         elif n == 2:
             possible_roles = ['none', 'visitor', 'participant', 'moderator']
-            return the_input.new_completion(possible_roles, 2, '',
+            return Completion(the_input.new_completion, possible_roles, 2, '',
                                             quotify=True)
 
     def completion_affiliation(self, the_input):
@@ -315,26 +306,26 @@ class MucTab(ChatTab):
             if self.core.xmpp.boundjid.bare in jidlist:
                 jidlist.remove(self.core.xmpp.boundjid.bare)
             userlist.extend(jidlist)
-            return the_input.new_completion(userlist, 1, '', quotify=True)
+            return Completion(the_input.new_completion, userlist, 1, '', quotify=True)
         elif n == 2:
             possible_affiliations = ['none', 'member', 'admin',
                                      'owner', 'outcast']
-            return the_input.new_completion(possible_affiliations, 2, '',
+            return Completion(the_input.new_completion, possible_affiliations, 2, '',
                                             quotify=True)
 
     @command_args_parser.quoted(1, 1, [''])
     def command_invite(self, args):
         """/invite <jid> [reason]"""
         if args is None:
-            return self.core.command_help('invite')
+            return self.core.command.help('invite')
         jid, reason = args
-        self.core.command_invite('%s %s "%s"' % (jid, self.name, reason))
+        self.core.command.invite('%s %s "%s"' % (jid, self.name, reason))
 
     def completion_invite(self, the_input):
         """Completion for /invite"""
         n = the_input.get_argument_position(quoted=True)
         if n == 1:
-            return the_input.new_completion(roster.jids(), 1, quotify=True)
+            return Completion(the_input.new_completion, roster.jids(), 1, quotify=True)
 
     def scroll_user_list_up(self):
         self.user_win.scroll_up()
@@ -352,27 +343,33 @@ class MucTab(ChatTab):
         /info <nick>
         """
         if args is None:
-            return self.core.command_help('info')
+            return self.core.command.help('info')
         nick = args[0]
         user = self.get_user_by_name(nick)
         if not user:
-            return self.core.information("Unknown user: %s" % nick)
+            return self.core.information("Unknown user: %s" % nick, "Error")
         theme = get_theme()
+        inf = '\x19' + dump_tuple(theme.COLOR_INFORMATION_TEXT) + '}'
         if user.jid:
-            user_jid = ' (\x19%s}%s\x19o)' % (
+            user_jid = '%s (\x19%s}%s\x19o%s)' % (
+                            inf,
                             dump_tuple(theme.COLOR_MUC_JID),
-                            user.jid)
+                            user.jid,
+                            inf)
         else:
             user_jid = ''
-        info = ('\x19%s}%s\x19o%s: show: \x19%s}%s\x19o, affiliation:'
-                ' \x19%s}%s\x19o, role: \x19%s}%s\x19o%s') % (
+        info = ('\x19%s}%s\x19o%s%s: show: \x19%s}%s\x19o%s, affiliation:'
+                ' \x19%s}%s\x19o%s, role: \x19%s}%s\x19o%s') % (
                         dump_tuple(user.color),
                         nick,
                         user_jid,
+                        inf,
                         dump_tuple(theme.color_show(user.show)),
                         user.show or 'Available',
+                        inf,
                         dump_tuple(theme.color_role(user.role)),
                         user.affiliation or 'None',
+                        inf,
                         dump_tuple(theme.color_role(user.role)),
                         user.role or 'None',
                         '\n%s' % user.status if user.status else '')
@@ -415,7 +412,23 @@ class MucTab(ChatTab):
         self.disconnect()
         self.user_win.pos = 0
         self.core.disable_private_tabs(self.name)
-        self.core.command_join('"%s/%s"' % (self.name, self.own_nick))
+        self.join()
+
+    def join(self):
+        """
+        Join the room
+        """
+        status = self.core.get_status()
+        if self.last_connection:
+            delta = datetime.now() - self.last_connection
+            seconds = delta.seconds + delta.days * 24 * 3600
+        else:
+            seconds = 0
+        muc.join_groupchat(self.core, self.name, self.own_nick,
+                           self.password,
+                           status=status.message,
+                           show=status.show,
+                           seconds=seconds)
 
     @command_args_parser.quoted(0, 1, [''])
     def command_recolor(self, args):
@@ -472,7 +485,7 @@ class MucTab(ChatTab):
         User "random" to attribute a random color.
         """
         if args is None:
-            return self.core.command_help('color')
+            return self.core.command.help('color')
         nick = args[0]
         color = args[1].lower()
         user = self.get_user_by_name(nick)
@@ -522,7 +535,7 @@ class MucTab(ChatTab):
                          res.get('os') or 'an unknown platform')
             self.core.information(version, 'Info')
         if args is None:
-            return self.core.command_help('version')
+            return self.core.command.help('version')
         nick = args[0]
         if nick in [user.nick for user in self.users]:
             jid = safeJID(self.name).bare
@@ -538,7 +551,7 @@ class MucTab(ChatTab):
         /nick <nickname>
         """
         if args is None:
-            return self.core.command_help('nick')
+            return self.core.command.help('nick')
         nick = args[0]
         if not self.joined:
             return self.core.information('/nick only works in joined rooms',
@@ -571,7 +584,7 @@ class MucTab(ChatTab):
             if arg:
                 msg = ('\x19%(color_spec)s}%(spec)s\x19%(info_col)s} '
                        'You (\x19%(color)s}%(nick)s\x19%(info_col)s})'
-                       ' left the chatroom'
+                       ' left the room'
                        ' (\x19o%(reason)s\x19%(info_col)s})') % {
                            'info_col': info_col, 'reason': arg,
                            'spec': char_quit, 'color': color,
@@ -581,7 +594,7 @@ class MucTab(ChatTab):
             else:
                 msg = ('\x19%(color_spec)s}%(spec)s\x19%(info_col)s} '
                        'You (\x19%(color)s}%(nick)s\x19%(info_col)s})'
-                       ' left the chatroom') % {
+                       ' left the room') % {
                            'info_col': info_col,
                            'spec': char_quit, 'color': color,
                            'color_spec': spec_col,
@@ -595,6 +608,8 @@ class MucTab(ChatTab):
             if self == self.core.current_tab():
                 self.refresh()
             self.core.doupdate()
+        else:
+            muc.leave_groupchat(self.core.xmpp, self.name, self.own_nick, arg)
 
     @command_args_parser.raw
     def command_close(self, msg):
@@ -602,7 +617,11 @@ class MucTab(ChatTab):
         /close [msg]
         """
         self.command_part(msg)
-        self.core.close_tab()
+        self.core.close_tab(self)
+
+    def on_close(self):
+        super().on_close()
+        self.command_part('')
 
     @command_args_parser.quoted(1, 1)
     def command_query(self, args):
@@ -610,7 +629,7 @@ class MucTab(ChatTab):
         /query <nick> [message]
         """
         if args is None:
-            return  self.core.command_help('query')
+            return  self.core.command.help('query')
         nick = args[0]
         r = None
         for user in self.users:
@@ -629,12 +648,22 @@ class MucTab(ChatTab):
         /topic [new topic]
         """
         if not subject:
+            info_text = dump_tuple(get_theme().COLOR_INFORMATION_TEXT)
+            norm_text = dump_tuple(get_theme().COLOR_NORMAL_TEXT)
+            if self.topic_from:
+                user = self.get_user_by_name(self.topic_from)
+                if user:
+                    user_text = dump_tuple(user.color)
+                    user_string = '\x19%s}(set by \x19%s}%s\x19%s})' % (
+                            info_text, user_text, user.nick, info_text)
+                else:
+                    user_string = self.topic_from
+            else:
+                user_string = ''
+
             self._text_buffer.add_message(
-                    "\x19%s}The subject of the room is: %s %s" %
-                        (dump_tuple(get_theme().COLOR_INFORMATION_TEXT),
-                         self.topic,
-                         '(set by %s)' % self.topic_from if self.topic_from
-                                                         else ''))
+                    "\x19%s}The subject of the room is: \x19%s}%s %s" %
+                        (info_text, norm_text, self.topic, user_string))
             self.refresh()
             return
 
@@ -647,11 +676,7 @@ class MucTab(ChatTab):
         """
         if not self.joined:
             return
-        color_visitor = dump_tuple(get_theme().COLOR_USER_VISITOR)
-        color_other = dump_tuple(get_theme().COLOR_USER_NONE)
-        color_moderator = dump_tuple(get_theme().COLOR_USER_MODERATOR)
-        color_participant = dump_tuple(get_theme().COLOR_USER_PARTICIPANT)
-        visitors, moderators, participants, others = [], [], [], []
+
         aff = {
                 'owner': get_theme().CHAR_AFFILIATION_OWNER,
                 'admin': get_theme().CHAR_AFFILIATION_ADMIN,
@@ -659,37 +684,20 @@ class MucTab(ChatTab):
                 'none': get_theme().CHAR_AFFILIATION_NONE,
                 }
 
-        users = self.users[:]
-        users.sort(key=lambda x: x.nick.lower())
-        for user in users:
-            color = aff.get(user.affiliation,
-                            get_theme().CHAR_AFFILIATION_NONE)
-            if user.role == 'visitor':
-                visitors.append((user, color))
-            elif user.role == 'participant':
-                participants.append((user, color))
-            elif user.role == 'moderator':
-                moderators.append((user, color))
-            else:
-                others.append((user, color))
+        colors = {}
+        colors["visitor"] = dump_tuple(get_theme().COLOR_USER_VISITOR)
+        colors["moderator"] = dump_tuple(get_theme().COLOR_USER_MODERATOR)
+        colors["participant"] = dump_tuple(get_theme().COLOR_USER_PARTICIPANT)
+        color_other = dump_tuple(get_theme().COLOR_USER_NONE)
 
         buff = ['Users: %s \n' % len(self.users)]
-        for moderator in moderators:
+        for user in self.users:
+            affiliation = aff.get(user.affiliation,
+                                  get_theme().CHAR_AFFILIATION_NONE)
+            color = colors.get(user.role, color_other)
             buff.append('\x19%s}%s\x19o\x19%s}%s\x19o' % (
-                    color_moderator, moderator[1],
-                    dump_tuple(moderator[0].color), moderator[0].nick))
-        for participant in participants:
-            buff.append('\x19%s}%s\x19o\x19%s}%s\x19o' % (
-                    color_participant, participant[1],
-                    dump_tuple(participant[0].color), participant[0].nick))
-        for visitor in visitors:
-            buff.append('\x19%s}%s\x19o\x19%s}%s\x19o' % (
-                    color_visitor, visitor[1],
-                    dump_tuple(visitor[0].color), visitor[0].nick))
-        for other in others:
-            buff.append('\x19%s}%s\x19o\x19%s}%s\x19o' % (
-                    color_other, other[1],
-                    dump_tuple(other[0].color), other[0].nick))
+                color, affiliation, dump_tuple(user.color), user.nick))
+
         buff.append('\n')
         message = ' '.join(buff)
 
@@ -699,7 +707,7 @@ class MucTab(ChatTab):
 
     def completion_topic(self, the_input):
         if the_input.get_argument_position() == 1:
-            return the_input.auto_completion([self.topic], '', quotify=False)
+            return Completion(the_input.auto_completion, [self.topic], '', quotify=False)
 
     def completion_quoted(self, the_input):
         """Nick completion, but with quotes"""
@@ -710,7 +718,7 @@ class MucTab(ChatTab):
                 if user.nick != self.own_nick:
                     word_list.append(user.nick)
 
-            return the_input.new_completion(word_list, 1, quotify=True)
+            return Completion(the_input.new_completion, word_list, 1, quotify=True)
 
     @command_args_parser.quoted(1, 1)
     def command_kick(self, args):
@@ -718,7 +726,7 @@ class MucTab(ChatTab):
         /kick <nick> [reason]
         """
         if args is None:
-            return self.core.command_help('kick')
+            return self.core.command.help('kick')
         if len(args) == 2:
             msg = ' "%s"' % args[1]
         else:
@@ -734,7 +742,7 @@ class MucTab(ChatTab):
             if iq['type'] == 'error':
                 self.core.room_error(iq, self.name)
         if args is None:
-            return self.core.command_help('ban')
+            return self.core.command.help('ban')
         if len(args) > 1:
             msg = args[1]
         else:
@@ -764,7 +772,7 @@ class MucTab(ChatTab):
                 self.core.room_error(iq, self.name)
 
         if args is None:
-            return self.core.command_help('role')
+            return self.core.command.help('role')
 
         nick, role, reason = args[0], args[1].lower(), args[2]
 
@@ -791,7 +799,7 @@ class MucTab(ChatTab):
                 self.core.room_error(iq, self.name)
 
         if args is None:
-            return self.core.command_help('affiliation')
+            return self.core.command.help('affiliation')
 
         nick, affiliation = args[0], args[1].lower()
 
@@ -866,7 +874,7 @@ class MucTab(ChatTab):
         /ignore <nick>
         """
         if args is None:
-            return self.core.command_help('ignore')
+            return self.core.command.help('ignore')
 
         nick = args[0]
         user = self.get_user_by_name(nick)
@@ -884,7 +892,7 @@ class MucTab(ChatTab):
         /unignore <nick>
         """
         if args is None:
-            return self.core.command_help('unignore')
+            return self.core.command.help('unignore')
 
         nick = args[0]
         user = self.get_user_by_name(nick)
@@ -899,7 +907,7 @@ class MucTab(ChatTab):
     def completion_unignore(self, the_input):
         if the_input.get_argument_position() == 1:
             users = [user.nick for user in self.ignores]
-            return the_input.auto_completion(users, quotify=False)
+            return Completion(the_input.auto_completion, users, quotify=False)
 
     def resize(self):
         """
@@ -907,18 +915,14 @@ class MucTab(ChatTab):
         """
         self.need_resize = False
         if config.get('hide_user_list') or self.size.tab_degrade_x:
-            display_user_list = False
             text_width = self.width
         else:
-            display_user_list = True
             text_width = (self.width // 10) * 9
 
         if self.size.tab_degrade_y:
-            display_info_win = False
             tab_win_height = 0
             info_win_height = 0
         else:
-            display_info_win = True
             tab_win_height = Tab.tab_win_height()
             info_win_height = self.core.information_win_size
 
@@ -958,7 +962,7 @@ class MucTab(ChatTab):
         if display_user_list:
             self.v_separator.refresh()
             self.user_win.refresh(self.users)
-        self.info_header.refresh(self, self.text_win)
+        self.info_header.refresh(self, self.text_win, user=self.own_user)
         self.refresh_tab_win()
         if display_info_win:
             self.info_win.refresh()
@@ -1069,7 +1073,7 @@ class MucTab(ChatTab):
         from_room = presence['from'].bare
         xpath = '{%s}x/{%s}status' % (NS_MUC_USER, NS_MUC_USER)
         status_codes = set()
-        for status_code in presence.findall(xpath):
+        for status_code in presence.xml.findall(xpath):
             status_codes.add(status_code.attrib['code'])
 
         # Check if it's not an error presence.
@@ -1082,13 +1086,13 @@ class MucTab(ChatTab):
         jid = presence['muc']['jid']
         typ = presence['type']
         deterministic = config.get_by_tabname('deterministic_nick_colors', self.name)
-        color = self.search_for_color(from_nick)
         if not self.joined:     # user in the room BEFORE us.
             # ignore redondant presence message, see bug #1509
             if (from_nick not in [user.nick for user in self.users]
                     and typ != "unavailable"):
+                user_color = self.search_for_color(from_nick)
                 new_user = User(from_nick, affiliation, show,
-                                status, role, jid, deterministic, color)
+                                status, role, jid, deterministic, user_color)
                 bisect.insort_left(self.users, new_user)
                 self.core.events.trigger('muc_join', presence, self)
                 if '110' in status_codes or self.own_nick == from_nick:
@@ -1096,6 +1100,7 @@ class MucTab(ChatTab):
                     # ejabberd or every gateway in the world that just do
                     # not send a 110 status code with the presence
                     self.own_nick = from_nick
+                    self.own_user = new_user
                     self.joined = True
                     if self.name in self.core.initial_joins:
                         self.core.initial_joins.remove(self.name)
@@ -1116,19 +1121,17 @@ class MucTab(ChatTab):
                     info_col = dump_tuple(get_theme().COLOR_INFORMATION_TEXT)
                     warn_col = dump_tuple(get_theme().COLOR_WARNING_TEXT)
                     spec_col = dump_tuple(get_theme().COLOR_JOIN_CHAR)
-
-                    self.add_message(
+                    enable_message = (
                         '\x19%(color_spec)s}%(spec)s\x19%(info_col)s} You '
                         '(\x19%(nick_col)s}%(nick)s\x19%(info_col)s}) joined'
-                        ' the chatroom' %
-                            {
+                        ' the room') % {
                             'nick': from_nick,
                             'spec': get_theme().CHAR_JOIN,
                             'color_spec': spec_col,
                             'nick_col': color,
                             'info_col': info_col,
-                            },
-                        typ=2)
+                            }
+                    self.add_message(enable_message, typ=2)
                     if '201' in status_codes:
                         self.add_message(
                                 '\x19%(info_col)s}Info: The room '
@@ -1153,7 +1156,7 @@ class MucTab(ChatTab):
                         self.refresh_tab_win()
                         self.core.current_tab().input.refresh()
                         self.core.doupdate()
-                    self.core.enable_private_tabs(self.name)
+                    self.core.enable_private_tabs(self.name, enable_message)
                     # Enable the self ping event, to regularly check if we
                     # are still in the room.
                     self.enable_self_ping_event()
@@ -1166,9 +1169,10 @@ class MucTab(ChatTab):
             user = self.get_user_by_name(from_nick)
             # New user
             if not user:
+                user_color = self.search_for_color(from_nick)
                 self.core.events.trigger('muc_join', presence, self)
                 self.on_user_join(from_nick, affiliation, show, status, role,
-                                  jid, color)
+                                  jid, user_color)
             # nick change
             elif change_nick:
                 self.core.events.trigger('muc_nickchange', presence, self)
@@ -1176,13 +1180,13 @@ class MucTab(ChatTab):
             elif ban:
                 self.core.events.trigger('muc_ban', presence, self)
                 self.core.on_user_left_private_conversation(from_room,
-                                                            from_nick, status)
+                                                            user, status)
                 self.on_user_banned(presence, user, from_nick)
             # kick
             elif kick:
                 self.core.events.trigger('muc_kick', presence, self)
                 self.core.on_user_left_private_conversation(from_room,
-                                                            from_nick, status)
+                                                            user, status)
                 self.on_user_kicked(presence, user, from_nick)
             elif shutdown:
                 self.core.events.trigger('muc_shutdown', presence, self)
@@ -1201,7 +1205,7 @@ class MucTab(ChatTab):
         if self.core.current_tab() is self:
             self.text_win.refresh()
             self.user_win.refresh_if_changed(self.users)
-            self.info_header.refresh(self, self.text_win)
+            self.info_header.refresh(self, self.text_win, user=self.own_user)
             self.input.refresh()
             self.core.doupdate()
 
@@ -1244,16 +1248,16 @@ class MucTab(ChatTab):
             char_join = get_theme().CHAR_JOIN
             if not jid.full:
                 msg = ('\x19%(color_spec)s}%(spec)s \x19%(color)s}%(nick)s'
-                       '\x19%(info_col)s} joined the chatroom') % {
+                       '\x19%(info_col)s} joined the room') % {
                             'nick': from_nick, 'spec': char_join,
                             'color': color,
                             'info_col': info_col,
                             'color_spec': spec_col,
                             }
             else:
-                msg = ('\x19%(color_spec)s}%(spec)s \x19%(color)s}%(nick)s '
-                       '\x19%(info_col)s}(\x19%(jid_color)s}%(jid)s\x19'
-                       '%(info_col)s}) joined the chatroom') % {
+                msg = ('\x19%(color_spec)s}%(spec)s \x19%(color)s}%(nick)s'
+                       '\x19%(info_col)s} (\x19%(jid_color)s}%(jid)s\x19'
+                       '%(info_col)s}) joined the room') % {
                             'spec': char_join, 'nick': from_nick,
                             'color':color, 'jid':jid.full,
                             'info_col': info_col,
@@ -1264,12 +1268,12 @@ class MucTab(ChatTab):
         self.core.on_user_rejoined_private_conversation(self.name, from_nick)
 
     def on_user_nick_change(self, presence, user, from_nick, from_room):
-        new_nick = presence.find('{%s}x/{%s}item' % (NS_MUC_USER, NS_MUC_USER)
-                                ).attrib['nick']
+        new_nick = presence.xml.find('{%s}x/{%s}item' % (NS_MUC_USER, NS_MUC_USER)
+                                     ).attrib['nick']
         if user.nick == self.own_nick:
             self.own_nick = new_nick
             # also change our nick in all private discussions of this room
-            self.core.on_muc_own_nickchange(self)
+            self.core.handler.on_muc_own_nickchange(self)
         else:
             color = config.get_by_tabname(new_nick, 'muc_colors')
             if color != '':
@@ -1292,17 +1296,17 @@ class MucTab(ChatTab):
                               'color':color, 'info_col': info_col},
                          typ=2)
         # rename the private tabs if needed
-        self.core.rename_private_tabs(self.name, from_nick, new_nick)
+        self.core.rename_private_tabs(self.name, from_nick, user)
 
     def on_user_banned(self, presence, user, from_nick):
         """
         When someone is banned from a muc
         """
         self.users.remove(user)
-        by = presence.find('{%s}x/{%s}item/{%s}actor' %
-                            (NS_MUC_USER, NS_MUC_USER, NS_MUC_USER))
-        reason = presence.find('{%s}x/{%s}item/{%s}reason' %
-                                (NS_MUC_USER, NS_MUC_USER, NS_MUC_USER))
+        by = presence.xml.find('{%s}x/{%s}item/{%s}actor' %
+                               (NS_MUC_USER, NS_MUC_USER, NS_MUC_USER))
+        reason = presence.xml.find('{%s}x/{%s}item/{%s}reason' %
+                                   (NS_MUC_USER, NS_MUC_USER, NS_MUC_USER))
         by = by.attrib['jid'] if by is not None else None
 
         info_col = dump_tuple(get_theme().COLOR_INFORMATION_TEXT)
@@ -1367,10 +1371,10 @@ class MucTab(ChatTab):
         When someone is kicked from a muc
         """
         self.users.remove(user)
-        actor_elem = presence.find('{%s}x/{%s}item/{%s}actor' %
-                                     (NS_MUC_USER, NS_MUC_USER, NS_MUC_USER))
-        reason = presence.find('{%s}x/{%s}item/{%s}reason' %
-                                (NS_MUC_USER, NS_MUC_USER, NS_MUC_USER))
+        actor_elem = presence.xml.find('{%s}x/{%s}item/{%s}actor' %
+                                       (NS_MUC_USER, NS_MUC_USER, NS_MUC_USER))
+        reason = presence.xml.find('{%s}x/{%s}item/{%s}reason' %
+                                   (NS_MUC_USER, NS_MUC_USER, NS_MUC_USER))
         by = None
         info_col = dump_tuple(get_theme().COLOR_INFORMATION_TEXT)
         char_kick = get_theme().CHAR_KICK
@@ -1457,7 +1461,7 @@ class MucTab(ChatTab):
             if not jid.full:
                 leave_msg = ('\x19%(color_spec)s}%(spec)s \x19%(color)s}'
                              '%(nick)s\x19%(info_col)s} has left the '
-                             'chatroom') % {
+                             'room') % {
                                   'nick':from_nick, 'color':color,
                                   'spec':get_theme().CHAR_QUIT,
                                   'info_col': info_col,
@@ -1467,7 +1471,7 @@ class MucTab(ChatTab):
                 leave_msg = ('\x19%(color_spec)s}%(spec)s \x19%(color)s}'
                              '%(nick)s\x19%(info_col)s} (\x19%(jid_col)s}'
                              '%(jid)s\x19%(info_col)s}) has left the '
-                             'chatroom') % {
+                             'room') % {
                                   'spec':get_theme().CHAR_QUIT,
                                   'nick':from_nick, 'color':color,
                                   'jid':jid.full, 'info_col': info_col,
@@ -1476,7 +1480,7 @@ class MucTab(ChatTab):
             if status:
                 leave_msg += ' (\x19o%s\x19%s})' % (status, info_col)
             self.add_message(leave_msg, typ=2)
-        self.core.on_user_left_private_conversation(from_room, from_nick,
+        self.core.on_user_left_private_conversation(from_room, user,
                                                     status)
 
     def on_user_change_status(
@@ -1539,7 +1543,7 @@ class MucTab(ChatTab):
             self._text_buffer.add_message(msg)
         self.core.on_user_changed_status_in_private('%s/%s' %
                                                       (from_room, from_nick),
-                                                    msg)
+                                                    Status(show, status))
         self.users.remove(user)
         # finally, effectively change the user status
         user.update(affiliation, show, status, role)
@@ -1572,12 +1576,12 @@ class MucTab(ChatTab):
                 self.core.information('Unable to write in the log file',
                                       'Error')
 
-    def do_highlight(self, txt, time, nickname):
+    def do_highlight(self, txt, time, nickname, corrected=False):
         """
         Set the tab color and returns the nick color
         """
         highlighted = False
-        if not time and nickname and nickname != self.own_nick and self.joined:
+        if (not time or corrected) and nickname and nickname != self.own_nick and self.joined:
 
             if re.search(r'\b' + self.own_nick.lower() + r'\b', txt.lower()):
                 if self.state != 'current':
@@ -1659,14 +1663,13 @@ class MucTab(ChatTab):
     def modify_message(self, txt, old_id, new_id,
                        time=None, nickname=None, user=None, jid=None):
         self.log_message(txt, nickname, time=time, typ=1)
-        highlight = self.do_highlight(txt, time, nickname)
+        highlight = self.do_highlight(txt, time, nickname, corrected=True)
         message = self._text_buffer.modify_message(txt, old_id, new_id,
                                                    highlight=highlight,
                                                    time=time, user=user,
                                                    jid=jid)
         if message:
             self.text_win.modify_message(old_id, message)
-            self.core.refresh_window()
             return highlight
         return False
 
