@@ -16,17 +16,19 @@ import ssl
 from os import getenv, path
 from functools import partial
 
-from . import Tab
+from poezio import common
+from poezio import windows
+from poezio.common import safeJID
+from poezio.config import config
+from poezio.contact import Contact, Resource
+from poezio.decorators import refresh_wrapper
+from poezio.roster import RosterGroup, roster
+from poezio.theming import get_theme, dump_tuple
+from poezio.decorators import command_args_parser
+from poezio.core.structs import Completion
+
+from poezio.tabs import Tab
 
-import common
-import windows
-from common import safeJID
-from config import config
-from contact import Contact, Resource
-from decorators import refresh_wrapper
-from roster import RosterGroup, roster
-from theming import get_theme, dump_tuple
-from decorators import command_args_parser
 
 class RosterInfoTab(Tab):
     """
@@ -34,8 +36,8 @@ class RosterInfoTab(Tab):
     """
     plugin_commands = {}
     plugin_keys = {}
-    def __init__(self):
-        Tab.__init__(self)
+    def __init__(self, core):
+        Tab.__init__(self, core)
         self.name = "Roster"
         self.v_separator = windows.VerticalSeparator()
         self.information_win = windows.TextWin()
@@ -70,38 +72,38 @@ class RosterInfoTab(Tab):
                     desc='Deny your presence to the provided JID (or the '
                          'selected contact in your roster), who is asking'
                          'you to be in his/here roster.',
-                    shortdesc='Deny an user your presence.',
+                    shortdesc='Deny a user your presence.',
                     completion=self.completion_deny)
             self.register_command('accept', self.command_accept,
                     usage='[jid]',
                     desc='Allow the provided JID (or the selected contact '
                          'in your roster), to see your presence.',
-                    shortdesc='Allow an user your presence.',
+                    shortdesc='Allow a user your presence.',
                     completion=self.completion_deny)
             self.register_command('add', self.command_add,
                     usage='<jid>',
                     desc='Add the specified JID to your roster, ask him to'
                          ' allow you to see his presence, and allow him to'
                          ' see your presence.',
-                    shortdesc='Add an user to your roster.')
+                    shortdesc='Add a user to your roster.')
             self.register_command('name', self.command_name,
                     usage='<jid> [name]',
                     shortdesc='Set the given JID\'s name.',
                     completion=self.completion_name)
             self.register_command('groupadd', self.command_groupadd,
-                    usage='<jid> <group>',
-                    desc='Add the given JID to the given group.',
-                    shortdesc='Add an user to a group',
+                    usage='[<jid> <group>]|<group>',
+                    desc='Add the given JID or selected line to the given group.',
+                    shortdesc='Add a user to a group',
                     completion=self.completion_groupadd)
             self.register_command('groupmove', self.command_groupmove,
                     usage='<jid> <old group> <new group>',
                     desc='Move the given JID from the old group to the new group.',
-                    shortdesc='Move an user to another group.',
+                    shortdesc='Move a user to another group.',
                     completion=self.completion_groupmove)
             self.register_command('groupremove', self.command_groupremove,
                     usage='<jid> <group>',
                     desc='Remove the given JID from the given group.',
-                    shortdesc='Remove an user from a group.',
+                    shortdesc='Remove a user from a group.',
                     completion=self.completion_groupremove)
             self.register_command('remove', self.command_remove,
                     usage='[jid]',
@@ -109,7 +111,7 @@ class RosterInfoTab(Tab):
                          'will unsubscribe you from its presence, cancel '
                          'its subscription to yours, and remove the item '
                          'from your roster.',
-                    shortdesc='Remove an user from your roster.',
+                    shortdesc='Remove a user from your roster.',
                     completion=self.completion_remove)
             self.register_command('export', self.command_export,
                     usage='[/path/to/file]',
@@ -140,7 +142,7 @@ class RosterInfoTab(Tab):
                 usage='<jid>',
                 desc='Informs you of the last activity of a JID.',
                 shortdesc='Get the activity of someone.',
-                completion=self.core.completion_last_activity)
+                completion=self.core.completion.last_activity)
 
         self.resize()
         self.update_commands()
@@ -230,7 +232,7 @@ class RosterInfoTab(Tab):
         /cert_add <name> <certfile> [cert-management]
         """
         if not args or len(args) < 2:
-            return self.core.command_help('cert_add')
+            return self.core.command.help('cert_add')
         def cb(iq):
             if iq['type'] == 'error':
                 self.core.information('Unable to add the certificate.', 'Error')
@@ -267,8 +269,6 @@ class RosterInfoTab(Tab):
         """
         completion for /cert_add <name> <path> [management]
         """
-        text = the_input.get_text()
-        args = common.shell_split(text)
         n = the_input.get_argument_position()
         log.debug('%s %s %s', the_input.text, n, the_input.pos)
         if n == 1:
@@ -276,7 +276,7 @@ class RosterInfoTab(Tab):
         elif n == 2:
             return self.completion_file(2, the_input)
         elif n == 3:
-            return the_input.new_completion(['true', 'false'], n)
+            return Completion(the_input.new_completion, ['true', 'false'], n)
 
     @command_args_parser.quoted(1)
     def command_cert_disable(self, args):
@@ -284,7 +284,7 @@ class RosterInfoTab(Tab):
         /cert_disable <name>
         """
         if not args:
-            return self.core.command_help('cert_disable')
+            return self.core.command.help('cert_disable')
         def cb(iq):
             if iq['type'] == 'error':
                 self.core.information('Unable to disable the certificate.', 'Error')
@@ -301,7 +301,7 @@ class RosterInfoTab(Tab):
         /cert_revoke <name>
         """
         if not args:
-            return self.core.command_help('cert_revoke')
+            return self.core.command.help('cert_revoke')
         def cb(iq):
             if iq['type'] == 'error':
                 self.core.information('Unable to revoke the certificate.', 'Error')
@@ -319,7 +319,7 @@ class RosterInfoTab(Tab):
         /cert_fetch <name> <path>
         """
         if not args or len(args) < 2:
-            return self.core.command_help('cert_fetch')
+            return self.core.command.help('cert_fetch')
         def cb(iq):
             if iq['type'] == 'error':
                 self.core.information('Unable to fetch the certificate.',
@@ -350,8 +350,6 @@ class RosterInfoTab(Tab):
         """
         completion for /cert_fetch <name> <path>
         """
-        text = the_input.get_text()
-        args = common.shell_split(text)
         n = the_input.get_argument_position()
         log.debug('%s %s %s', the_input.text, n, the_input.pos)
         if n == 1:
@@ -398,7 +396,7 @@ class RosterInfoTab(Tab):
         """
         if the_input.get_argument_position() == 1:
             jids = roster.jids()
-            return the_input.new_completion(jids, 1, '', quotify=False)
+            return Completion(the_input.new_completion, jids, 1, '', quotify=False)
 
     @command_args_parser.quoted(0, 1)
     def command_unblock(self, args):
@@ -428,7 +426,7 @@ class RosterInfoTab(Tab):
             if iq['type'] == 'error':
                 return
             l = sorted(str(item) for item in iq['blocklist']['items'])
-            return the_input.new_completion(l, 1, quotify=False)
+            return Completion(the_input.new_completion, l, 1, quotify=False)
 
         if the_input.get_argument_position():
             self.core.xmpp.plugin['xep_0191'].get_blocked(callback=on_result)
@@ -485,7 +483,7 @@ class RosterInfoTab(Tab):
         else:
             self.core.information('No JID selected.', 'Error')
             return
-        self.core.command_last_activity(jid)
+        self.core.command.last_activity(jid)
 
     def resize(self):
         self.need_resize = False
@@ -500,7 +498,7 @@ class RosterInfoTab(Tab):
                 contact_win_h = 0
             else:
                 display_contact_win = True
-                contact_win_h = 5
+                contact_win_h = 4
         if self.size.tab_degrade_y:
             tab_win_height = 0
         else:
@@ -515,10 +513,10 @@ class RosterInfoTab(Tab):
                                         info_width, 0, roster_width + 1,
                                         self.core.information_buffer)
             if display_contact_win:
-                self.contact_info_win.resize(contact_win_h - tab_win_height,
+                self.contact_info_win.resize(contact_win_h,
                                              info_width,
                                              self.height - tab_win_height
-                                             - contact_win_h,
+                                             - contact_win_h - 1,
                                              roster_width + 1)
         self.roster_win.resize(self.height - 1 - Tab.tab_win_height(),
                                roster_width, 0, 0)
@@ -543,7 +541,7 @@ class RosterInfoTab(Tab):
         if n == complete_number:
             if args[n-1] == '' or len(args) < n+1:
                 home = os.getenv('HOME') or '/'
-                return the_input.new_completion([home, '/tmp'], n, quotify=True)
+                return Completion(the_input.new_completion, [home, '/tmp'], n, quotify=True)
             path_ = args[n]
             if path.isdir(path_):
                 dir_ = path_
@@ -566,7 +564,7 @@ class RosterInfoTab(Tab):
                 if not name.startswith('.'):
                     end_list.append(value)
 
-            return the_input.new_completion(end_list, n, quotify=True)
+            return Completion(the_input.new_completion, end_list, n, quotify=True)
 
     @command_args_parser.ignored
     def command_clear(self):
@@ -603,12 +601,12 @@ class RosterInfoTab(Tab):
             if isinstance(item, Contact):
                 jid = item.bare_jid
             else:
-                self.core.information('No subscription to deny')
+                self.core.information('No subscription to deny', 'Warning')
                 return
         else:
             jid = safeJID(args[0]).bare
             if not jid in [jid for jid in roster.jids()]:
-                self.core.information('No subscription to deny')
+                self.core.information('No subscription to deny', 'Warning')
                 return
 
         contact = roster[jid]
@@ -646,7 +644,7 @@ class RosterInfoTab(Tab):
                 self.core.information('The name could not be set.', 'Error')
                 log.debug('Error in /name:\n%s', iq)
         if args is None:
-            return self.core.command_help('name')
+            return self.core.command.help('name')
         jid = safeJID(args[0]).bare
         name = args[1] if len(args) == 2 else ''
 
@@ -662,15 +660,25 @@ class RosterInfoTab(Tab):
         self.core.xmpp.update_roster(jid, name=name, groups=groups,
                                      subscription=subscription, callback=callback)
 
-    @command_args_parser.quoted(2)
+    @command_args_parser.quoted(1, 1)
     def command_groupadd(self, args):
         """
         Add the specified JID to the specified group
         """
         if args is None:
-            return self.core.command_help('groupadd')
-        jid = safeJID(args[0]).bare
-        group = args[1]
+            return self.core.command.help('groupadd')
+        if len(args) == 1:
+            group = args[0]
+            item = self.roster_win.selected_row
+            if isinstance(item, Contact):
+                jid = item.bare_jid
+            elif isinstance(item, Resource):
+                jid = item.jid
+            else:
+                return self.core.command.help('groupadd')
+        else:
+            jid = safeJID(args[0]).bare
+            group = args[1]
 
         contact = roster[jid]
         if contact is None:
@@ -708,7 +716,7 @@ class RosterInfoTab(Tab):
         Remove the specified JID from the first specified group and add it to the second one
         """
         if args is None:
-            return self.core.command_help('groupmove')
+            return self.core.command.help('groupmove')
         jid = safeJID(args[0]).bare
         group_from = args[1]
         group_to = args[2]
@@ -751,7 +759,7 @@ class RosterInfoTab(Tab):
             if iq:
                 roster.update_contact_groups(contact)
             else:
-                self.core.information('The group could not be set')
+                self.core.information('The group could not be set', 'Error')
                 log.debug('Error in groupmove:\n%s', iq)
 
         self.core.xmpp.update_roster(jid, name=name, groups=new_groups,
@@ -763,7 +771,7 @@ class RosterInfoTab(Tab):
         Remove the specified JID from the specified group
         """
         if args is None:
-            return self.core.command_help('groupremove')
+            return self.core.command.help('groupremove')
 
         jid = safeJID(args[0]).bare
         group = args[1]
@@ -811,7 +819,7 @@ class RosterInfoTab(Tab):
             if isinstance(item, Contact):
                 jid = item.bare_jid
             else:
-                self.core.information('No roster item to remove')
+                self.core.information('No roster item to remove', 'Error')
                 return
         roster.remove(jid)
         del roster[jid]
@@ -871,24 +879,24 @@ class RosterInfoTab(Tab):
         Completion for /remove
         """
         jids = [jid for jid in roster.jids()]
-        return the_input.auto_completion(jids, '', quotify=False)
+        return Completion(the_input.auto_completion, jids, '', quotify=False)
 
     def completion_name(self, the_input):
         """Completion for /name"""
         n = the_input.get_argument_position()
         if n == 1:
             jids = [jid for jid in roster.jids()]
-            return the_input.new_completion(jids, n, quotify=True)
+            return Completion(the_input.new_completion, jids, n, quotify=True)
         return False
 
     def completion_groupadd(self, the_input):
         n = the_input.get_argument_position()
         if n == 1:
             jids = sorted(jid for jid in roster.jids())
-            return the_input.new_completion(jids, n, '', quotify=True)
+            return Completion(the_input.new_completion, jids, n, '', quotify=True)
         elif n == 2:
             groups = sorted(group for group in roster.groups if group != 'none')
-            return the_input.new_completion(groups, n, '', quotify=True)
+            return Completion(the_input.new_completion, groups, n, '', quotify=True)
         return False
 
     def completion_groupmove(self, the_input):
@@ -896,7 +904,7 @@ class RosterInfoTab(Tab):
         n = the_input.get_argument_position()
         if n == 1:
             jids = sorted(jid for jid in roster.jids())
-            return the_input.new_completion(jids, n, '', quotify=True)
+            return Completion(the_input.new_completion, jids, n, '', quotify=True)
         elif n == 2:
             contact = roster[args[1]]
             if not contact:
@@ -904,10 +912,10 @@ class RosterInfoTab(Tab):
             groups = list(contact.groups)
             if 'none' in groups:
                 groups.remove('none')
-            return the_input.new_completion(groups, n, '', quotify=True)
+            return Completion(the_input.new_completion, groups, n, '', quotify=True)
         elif n == 3:
             groups = sorted(group for group in roster.groups)
-            return the_input.new_completion(groups, n, '', quotify=True)
+            return Completion(the_input.new_completion, groups, n, '', quotify=True)
         return False
 
     def completion_groupremove(self, the_input):
@@ -915,7 +923,7 @@ class RosterInfoTab(Tab):
         n = the_input.get_argument_position()
         if n == 1:
             jids = sorted(jid for jid in roster.jids())
-            return the_input.new_completion(jids, n, '', quotify=True)
+            return Completion(the_input.new_completion, jids, n, '', quotify=True)
         elif n == 2:
             contact = roster[args[1]]
             if contact is None:
@@ -925,7 +933,7 @@ class RosterInfoTab(Tab):
                 groups.remove('none')
             except ValueError:
                 pass
-            return the_input.new_completion(groups, n, '', quotify=True)
+            return Completion(the_input.new_completion, groups, n, '', quotify=True)
         return False
 
     def completion_deny(self, the_input):
@@ -935,7 +943,7 @@ class RosterInfoTab(Tab):
         """
         jids = sorted(str(contact.bare_jid) for contact in roster.contacts.values()
              if contact.pending_in)
-        return the_input.new_completion(jids, 1, '', quotify=False)
+        return Completion(the_input.new_completion, jids, 1, '', quotify=False)
 
     @command_args_parser.quoted(0, 1)
     def command_accept(self, args):
@@ -947,7 +955,7 @@ class RosterInfoTab(Tab):
             if isinstance(item, Contact):
                 jid = item.bare_jid
             else:
-                self.core.information('No subscription to accept')
+                self.core.information('No subscription to accept', 'Warning')
                 return
         else:
             jid = safeJID(args[0]).bare
@@ -1017,8 +1025,6 @@ class RosterInfoTab(Tab):
         """
         '/' is pressed, we enter "input mode"
         """
-        if isinstance(self.input, windows.YesNoInput):
-            return
         curses.curs_set(1)
         self.input = windows.CommandInput("", self.reset_help_message, self.execute_slash_command)
         self.input.resize(1, self.width, self.height-1, 0)
@@ -1125,9 +1131,9 @@ class RosterInfoTab(Tab):
         selected_row = self.roster_win.get_selected_row()
         if isinstance(selected_row, Contact):
             for resource in selected_row.resources:
-                self.core.command_version(str(resource.jid))
+                self.core.command.version(str(resource.jid))
         elif isinstance(selected_row, Resource):
-            self.core.command_version(str(selected_row.jid))
+            self.core.command.version(str(selected_row.jid))
         else:
             self.core.information('Nothing to get versions from', 'Info')
 
@@ -1194,8 +1200,6 @@ class RosterInfoTab(Tab):
         Start the search. The input should appear with a short instruction
         in it.
         """
-        if isinstance(self.input, windows.YesNoInput):
-            return
         curses.curs_set(1)
         self.input = windows.CommandInput("[Search]", self.on_search_terminate, self.on_search_terminate, self.set_roster_filter)
         self.input.resize(1, self.width, self.height-1, 0)
@@ -1206,8 +1210,6 @@ class RosterInfoTab(Tab):
 
     @refresh_wrapper.always
     def start_search_slow(self):
-        if isinstance(self.input, windows.YesNoInput):
-            return
         curses.curs_set(1)
         self.input = windows.CommandInput("[Search]", self.on_search_terminate, self.on_search_terminate, self.set_roster_filter_slow)
         self.input.resize(1, self.width, self.height-1, 0)
