@@ -3,8 +3,9 @@ Standalone functions used by the modules
 """
 
 import string
+DIGITS = string.digits + '-'
 
-from . base_wins import FORMAT_CHAR, format_chars
+from poezio.windows.base_wins import FORMAT_CHAR, format_chars
 
 def find_first_format_char(text, chars=None):
     if chars is None:
@@ -35,14 +36,14 @@ def parse_attrs(text, previous=None):
         if next_attr_char + 1 < len(text):
             attr_char = text[next_attr_char+1].lower()
         else:
-            attr_char = str()
+            attr_char = '\0'
         if attr_char == 'o':
             attrs = []
         elif attr_char == 'u':
             attrs.append('u')
         elif attr_char == 'b':
             attrs.append('b')
-        if attr_char in string.digits and attr_char != '':
+        if attr_char in DIGITS and attr_char:
             color_str = text[next_attr_char+1:text.find('}', next_attr_char)]
             if color_str:
                 attrs.append(color_str + '}')
