@@ -8,14 +8,13 @@ log = logging.getLogger(__name__)
 import curses
 import string
 
-import keyboard
-import common
-import poopt
-from . import Win
-from . base_wins import format_chars
-from . funcs import find_first_format_char
-from config import config
-from theming import to_curses_attr
+from poezio import keyboard
+from poezio import common
+from poezio import poopt
+from poezio.windows.base_wins import Win, format_chars
+from poezio.windows.funcs import find_first_format_char
+from poezio.config import config
+from poezio.theming import to_curses_attr
 
 
 class Input(Win):
@@ -30,7 +29,7 @@ class Input(Win):
     passing the list of items that can be used to complete. The completion can be used
     in a very flexible way.
     """
-    text_attributes = ['b', 'o', 'u', '1', '2', '3', '4', '5', '6', '7', 't']
+    text_attributes = 'bou1234567t'
     clipboard = '' # A common clipboard for all the inputs, this makes
     # it easy cut and paste text between various input
     def __init__(self):
@@ -48,7 +47,7 @@ class Input(Win):
             '^W': self.delete_word,
             'M-d': self.delete_next_word,
             '^K': self.delete_end_of_line,
-            '^U': self.delete_begining_of_line,
+            '^U': self.delete_beginning_of_line,
             '^Y': self.paste_clipboard,
             '^A': self.key_home,
             '^E': self.key_end,
@@ -154,9 +153,9 @@ class Input(Win):
         self.key_end()
         return True
 
-    def delete_begining_of_line(self):
+    def delete_beginning_of_line(self):
         """
-        Cut the text from cursor to the begining of line
+        Cut the text from cursor to the beginning of line
         """
         if self.pos == 0:
             return True
@@ -189,7 +188,7 @@ class Input(Win):
 
     def key_home(self):
         """
-        Go to the begining of line
+        Go to the beginning of line
         """
         self.reset_completion()
         self.pos = 0
@@ -239,7 +238,7 @@ class Input(Win):
         """
         self.reset_completion()
         if self.pos == 0:
-            return
+            return False
         self.key_left()
         self.key_dc()
         return True
@@ -457,14 +456,13 @@ class Input(Win):
         """
         return self.text
 
-    def addstr_colored_lite(self, text, y=None, x=None):
+    def _addstr_colored_lite(self, text, y=None, x=None):
         """
         Just like addstr_colored, with the single-char attributes
         (\x0E to \x19 instead of \x19 + attr). We do not use any }
         char in this version
         """
-        chars = format_chars[:]
-        chars.append('\n')
+        chars = format_chars + '\n'
         if y is not None and x is not None:
             self.move(y, x)
         format_char = find_first_format_char(text, chars)
@@ -492,7 +490,7 @@ class Input(Win):
         """
         Refresh the line onscreen, but first, always adjust the
         view_pos.  Also, each FORMAT_CHAR+attr_char count only take
-        one screen column (this is done in addstr_colored_lite), we
+        one screen column (this is done in _addstr_colored_lite), we
         have to do some special calculations to find the correct
         length of text to display, and the position of the cursor.
         """
@@ -503,7 +501,7 @@ class Input(Win):
             self._win.attron(to_curses_attr(self.color))
         displayed_text = text[self.view_pos:self.view_pos+self.width-1].replace('\t', '\x18')
         self._win.attrset(0)
-        self.addstr_colored_lite(displayed_text)
+        self._addstr_colored_lite(displayed_text)
         # Fill the rest of the line with the input color
         if self.color:
             (_, x) = self._win.getyx()
@@ -541,9 +539,6 @@ class Input(Win):
         if poopt.wcswidth(self.text) < self.width:
             self.view_pos = 0
 
-        assert(self.pos >= self.view_pos and
-               self.pos <= self.view_pos + max(self.width, 3))
-
     def refresh(self):
         log.debug('Refresh: %s', self.__class__.__name__)
         self.rewrite_text()
@@ -568,6 +563,7 @@ class HistoryInput(Input):
     def __init__(self):
         Input.__init__(self)
         self.help_message = ''
+        self.histo_pos = -1
         self.current_completed = ''
         self.key_func['^R'] = self.toggle_search
         self.search = False
@@ -652,7 +648,6 @@ class MessageInput(HistoryInput):
     def __init__(self):
         HistoryInput.__init__(self)
         self.last_completion = None
-        self.histo_pos = -1
         self.key_func["KEY_UP"] = self.key_up
         self.key_func["M-A"] = self.key_up
         self.key_func["KEY_DOWN"] = self.key_down
@@ -707,10 +702,9 @@ class CommandInput(HistoryInput):
         self.key_func["M-A"] = self.key_up
         self.key_func["KEY_DOWN"] = self.key_down
         self.key_func["M-B"] = self.key_down
-        self.histo_pos = -1
 
-    def do_command(self, key, refresh=True, raw=False):
-        res = Input.do_command(self, key, refresh, raw)
+    def do_command(self, key, reset=True, raw=False):
+        res = Input.do_command(self, key, reset=reset, raw=raw)
         if self.on_input:
             self.on_input(self.get_text())
         return res
