@@ -186,14 +186,15 @@ import curses
 from potr.context import NotEncryptedError, UnencryptedMessage, ErrorReceived, NotOTRMessage,\
         STATE_ENCRYPTED, STATE_PLAINTEXT, STATE_FINISHED, Context, Account, crypt
 
-import common
-import xhtml
-from common import safeJID
-from config import config
-from plugin import BasePlugin
-from tabs import ConversationTab, DynamicConversationTab, PrivateTab
-from theming import get_theme, dump_tuple
-from decorators import command_args_parser
+from poezio import common
+from poezio import xhtml
+from poezio.common import safeJID
+from poezio.config import config
+from poezio.plugin import BasePlugin
+from poezio.tabs import ConversationTab, DynamicConversationTab, PrivateTab
+from poezio.theming import get_theme, dump_tuple
+from poezio.decorators import command_args_parser
+from poezio.core.structs import Completion
 
 OTR_DIR = os.path.join(os.getenv('XDG_DATA_HOME') or
                        '~/.local/share', 'poezio', 'otr')
@@ -346,7 +347,10 @@ class PoezioContext(Context):
         message = self.xmpp.make_message(mto=self.peer,
                                          mbody=msg.decode('ascii'),
                                          mtype='chat')
+        message['eme']['namespace'] = 'urn:xmpp:otr:0'
         message.enable('carbon_private')
+        message.enable('no-copy')
+        message.enable('no-permanent-store')
         message.send()
 
     def setState(self, newstate):
@@ -490,6 +494,8 @@ class Plugin(BasePlugin):
         ConversationTab.add_information_element('otr', self.display_encryption_status)
         PrivateTab.add_information_element('otr', self.display_encryption_status)
 
+        self.core.xmpp.plugin['xep_0030'].add_feature('urn:xmpp:otr:0')
+
         self.account = PoezioAccount(self.core.xmpp.boundjid.bare, OTR_DIR)
         self.account.load_trusts()
         self.contexts = {}
@@ -528,6 +534,8 @@ class Plugin(BasePlugin):
         for context in self.contexts.values():
             context.disconnect()
 
+        self.core.xmpp.plugin['xep_0030'].del_feature(feature='urn:xmpp:otr:0')
+
         ConversationTab.remove_information_element('otr')
         PrivateTab.remove_information_element('otr')
 
@@ -535,17 +543,17 @@ class Plugin(BasePlugin):
         """
         Retrieve or create an OTR context
         """
-        jid = safeJID(jid).full
-        if not jid in self.contexts:
+        jid = safeJID(jid)
+        if not jid.full in self.contexts:
             flags = POLICY_FLAGS.copy()
             require = self.config.get_by_tabname('require_encryption',
-                                                 jid, default=False)
+                                                 jid.bare, default=False)
             flags['REQUIRE_ENCRYPTION'] = require
-            logging_policy = self.config.get_by_tabname('log', jid, default='false').lower()
-            self.contexts[jid] = PoezioContext(self.account, jid, self.core.xmpp, self.core)
-            self.contexts[jid].log = 1 if logging_policy != 'false' else 0
-            self.contexts[jid].flags = flags
-        return self.contexts[jid]
+            logging_policy = self.config.get_by_tabname('log', jid.bare , default=False)
+            self.contexts[jid.full] = PoezioContext(self.account, jid.full, self.core.xmpp, self.core)
+            self.contexts[jid.full].log = 1 if logging_policy else 0
+            self.contexts[jid.full].flags = flags
+        return self.contexts[jid.full]
 
     def on_conversation_msg(self, msg, tab):
         """
@@ -756,11 +764,16 @@ class Plugin(BasePlugin):
             'jid': name,
         }
 
-        ctx = self.contexts.get(name)
+        ctx = None
+        default_ctx = self.get_context(name)
+
         if isinstance(tab, DynamicConversationTab) and not tab.locked_resource:
             log.debug('Unlocked tab %s found, falling back to the first encrypted chat we find.', name)
             ctx = self.find_encrypted_context_with_matching(jid.bare)
 
+        if ctx is None:
+            ctx = default_ctx
+
         if ctx and ctx.state == STATE_ENCRYPTED:
             ctx.sendMessage(0, msg['body'].encode('utf-8'))
             if not tab.send_chat_state('active'):
@@ -803,7 +816,7 @@ class Plugin(BasePlugin):
         """
         args = common.shell_split(arg)
         if not args:
-            return self.core.command_help('otr')
+            return self.core.command.help('otr')
         action = args.pop(0)
         tab = self.api.current_tab()
         name = tab.name
@@ -905,7 +918,7 @@ class Plugin(BasePlugin):
         Completion for /otr
         """
         comp = ['start', 'fpr', 'ourfpr', 'refresh', 'end', 'trust', 'untrust']
-        return the_input.new_completion(comp, 1, quotify=False)
+        return Completion(the_input.new_completion, comp, 1, quotify=False)
 
     @command_args_parser.quoted(1, 2)
     def command_smp(self, args):
@@ -913,7 +926,7 @@ class Plugin(BasePlugin):
         /otrsmp <ask|answer|abort> [question] [secret]
         """
         if args is None or not args:
-            return self.core.command_help('otrsmp')
+            return self.core.command.help('otrsmp')
         length = len(args)
         action = args.pop(0)
         if length == 2:
@@ -965,7 +978,7 @@ class Plugin(BasePlugin):
     def completion_smp(the_input):
         """Completion for /otrsmp"""
         if the_input.get_argument_position() == 1:
-            return the_input.new_completion(['ask', 'answer', 'abort'], 1, quotify=False)
+            return Completion(the_input.new_completion, ['ask', 'answer', 'abort'], 1, quotify=False)
 
 def get_tlv(tlvs, cls):
     """Find the instance of a class in a list"""
